clc;close all;clear all;
disp('====================================================================================================================================');
author = 'Professor Pawel Szczepankowski,\nDepartment of Power Electronics and Electrical Machines, Faculty of Electrical and Control Engineering\nGdansk University of Technology GUT\n\n';
fprintf(author);
disp('This animation is a supplement to the article:');
disp('A new approach to the PWM modulation for the double, square–type, conventional matrix converters supplying loads with open–end winding.');
disp('This is an animation of the four basic variants of PWM modulation for the Conventional Matrix Converter');
disp('Go to the line 51 and change the variant of PWM modulation: rotation of reference output voltage vector for both converters.');
disp('Scripts present four variants of the Pulse Width Modulation for the Double Square type Multiphase Conventional Matrix Converters (DSM--CMC) supplying loads with the open--end winding.'); 
disp('====================================================================================================================================');

 
N=5;%number of inputs
M=5;%number of outputs
tend=0.1;%simulation end time
ts=250e-6;%simulation step
t=0:ts:tend;%time
len=length(t);%length of samples

fi=50;%frequency of the input voltage source in [Hz]
fo=25;%frequency of the reference output voltage in [Hz]
wi=2*pi*fi;%input pulsation
wo=2*pi*fo;%input pulsation
phasei=2*pi/N;%default input displacement
phaseo=2*pi/M;%default output displacement
anglei=wi*t;%set of the input angle
angleo=wo*t;%set of the output angle

%generation of the input voltage vectors collection
%using the Hilbert analytic signal concept
vi1=zeros(len,2);vi2=zeros(len,2);vi3=zeros(len,2);vi4=zeros(len,2);vi5=zeros(len,2);
Vi=[1 1 1 1 1];%input voltage amplitudes
for k=1:len
	vi1(k,:)=Vi(1)*[cos(anglei(k)-0*phasei) sin(anglei(k)-0*phasei)];
    vi2(k,:)=Vi(2)*[cos(anglei(k)-1*phasei) sin(anglei(k)-1*phasei)];
    vi3(k,:)=Vi(3)*[cos(anglei(k)-2*phasei) sin(anglei(k)-2*phasei)];
    vi4(k,:)=Vi(4)*[cos(anglei(k)-3*phasei) sin(anglei(k)-3*phasei)];
    vi5(k,:)=Vi(5)*[cos(anglei(k)-4*phasei) sin(anglei(k)-4*phasei)];
end

%generation of the output voltage vectors collection
%using the Hilbert analytic signal concept
vo1p=zeros(len,2);vo2p=zeros(len,2);vo3p=zeros(len,2);vo4p=zeros(len,2);vo5p=zeros(len,2);
vo1n=zeros(len,2);vo2n=zeros(len,2);vo3n=zeros(len,2);vo4n=zeros(len,2);vo5n=zeros(len,2);
%there are two converters: CMC_P and CMC_N
%to produce the maximum load voltage, the referenced output voltages  are generated in the opposite phase
q=cos(pi/5);
Voxp=q*ones(1,5);
Voxn=-Voxp;
%however, the imaginary part can be generated in a different manner
 variant='CCV--CV';
 %switch modulation variants
switch  variant
    case {'CV--CV'}
        Voyp=-q*ones(1,5);
        Voyn=q*ones(1,5);
        disp('PWM modulation variant CV--CV has been chosen.');
    case {'CCV--CCV'}
        Voyp=q*ones(1,5);
        Voyn=-q*ones(1,5);
        disp('PWM modulation variant CCV--CCV has been chosen.');
    case {'CV--CCV'}
        Voyp=-q*ones(1,5);
        Voyn=-q*ones(1,5);
        disp('PWM modulation variant CV--CVV has been chosen.');
    case {'CCV--CV'}
        Voyp=q*ones(1,5);
        Voyn=q*ones(1,5);
        disp('PWM modulation variant CCV--CV has been chosen.');
    otherwise
    disp('Unknown method.')
end
%generation of reference output vector coordinates
for k=1:len
    %p converter (the load left terminals)
    vo1p(k,:)=[Voxp(1)*cos(angleo(k)-0*phaseo) Voyp(1)*sin(angleo(k)-0*phaseo)];
    vo2p(k,:)=[Voxp(2)*cos(angleo(k)-1*phaseo) Voyp(2)*sin(angleo(k)-1*phaseo)];
    vo3p(k,:)=[Voxp(3)*cos(angleo(k)-2*phaseo) Voyp(3)*sin(angleo(k)-2*phaseo)];
    vo4p(k,:)=[Voxp(4)*cos(angleo(k)-3*phaseo) Voyp(4)*sin(angleo(k)-3*phaseo)];
    vo5p(k,:)=[Voxp(5)*cos(angleo(k)-4*phaseo) Voyp(5)*sin(angleo(k)-4*phaseo)];    
    %n converter (the load right terminals)
    vo1n(k,:)=[Voxn(1)*cos(angleo(k)-0*phaseo) Voyn(1)*sin(angleo(k)-0*phaseo)];
    vo2n(k,:)=[Voxn(2)*cos(angleo(k)-1*phaseo) Voyn(2)*sin(angleo(k)-1*phaseo)];
    vo3n(k,:)=[Voxn(3)*cos(angleo(k)-2*phaseo) Voyn(3)*sin(angleo(k)-2*phaseo)];
    vo4n(k,:)=[Voxn(4)*cos(angleo(k)-3*phaseo) Voyn(4)*sin(angleo(k)-3*phaseo)];
    vo5n(k,:)=[Voxn(5)*cos(angleo(k)-4*phaseo) Voyn(5)*sin(angleo(k)-4*phaseo)];
end

%average output voltage - memory prelocation
vo_avg_p=zeros(len,M);%"P" CMC converter average output voltage
vo_avg_n=zeros(len,M);%"N" CMC converter average output voltage
vo_avg=zeros(len,N);%average the load voltage
%average load current - memory prelocation
io=zeros(len,M);
%average input current - memory prelocation
ii_avg_p=zeros(len,N);%"P" CMC input average current
ii_avg_n=zeros(len,N);%"N" CMC input average current
ii_avg=zeros(len,N);%average input current
%algorithm - memory prelocation
index_min_p=zeros(len,M);
vertex1_p=zeros(len,3);
vertex2_p=zeros(len,3);
vertex3_p=zeros(len,3);
index_min_n=zeros(len,M);
vertex1_n=zeros(len,3);
vertex2_n=zeros(len,3);
vertex3_n=zeros(len,3);
d1p=zeros(len,M);
d2p=zeros(len,M);
d3p=zeros(len,M);
d1n=zeros(len,M);
d2n=zeros(len,M);
d3n=zeros(len,M);
iiavg_p=zeros(len,N);
iiavg_n=zeros(len,N);

%%%%%%%%%%%%%%%%%%%%% figures init starts here%%%%%%%%%%%%%%%%%%%%%%%%%%%
fig=figure('units','normalized','outerposition',[0 0 0.9 0.9]);
name='Direct Modulation concept for Conventional Matrix Converter with 5 inputs and 5 outputs: figure 1/2';
set(fig,'Color','white','MenuBar','figure','Name',name,'NumberTitle','off');
%left subfigure
subplot(2,2,[1 3]);
title({'$\rm{CMC_P}$ converter: $v_{\rm{i1}},...,v_{\rm{i5}}$ input voltage vectors '},{'$v_{\rm{o1p}}$ reference "p" vector'},'FontSize',18,'Interpreter','Latex'); 
axis equal;box on;hold on;
xlim([-1.1 1.1]);ylim([-1.1 1.1]);grid on;
xlabel('{x}','FontSize',24,'FontName','Times New Roman');
ylabel('{y}','FontSize',24,'Rotation',0,'FontName','Times New Roman');
set(get(gcf,'CurrentAxes'),'FontName','Times New Roman','FontSize',24,'XTick',[-1.0 -0.5 0 0.5 1.0],'YTick',[-1 -0.5 0 0.5 1]);
%right subfigure
subplot(2,2,[2 4]);
title({'$\rm{CMC_N}$ converter: $v_{\rm{i1}},...,v_{\rm{i5}}$ input voltage vectors '},{'$v_{\rm{o1n}}$ reference "n" vector'},'FontSize',18,'Interpreter','Latex'); 
axis equal;box on;hold on;
xlim([-1.1 1.1]);ylim([-1.1 1.1]);grid on;
xlabel('{x}','FontSize',24,'FontName','Times New Roman');
ylabel('{y}','FontSize',24,'Rotation',0,'FontName','Times New Roman');
set(get(gcf,'CurrentAxes'),'FontName','Times New Roman','FontSize',24,'XTick',[-1.0 -0.5 0 0.5 1.0],'YTick',[-1 -0.5 0 0.5 1]);
z =[0 0];%only for arrow3 function, center of the coordinate system
%%%%%%%%%%%%%%%%%%%%% figures init ends here%%%%%%%%%%%%%%%%%%%%%%%%%%%


%animation
for k=1:len
    
    %figure on the left
    subplot(2,2,[1 3]);    
    %collection of input vectors
    line_vi1=arrow3(z,vi1(k,:));line_vi2=arrow3(z,vi2(k,:));line_vi3=arrow3(z,vi3(k,:));
    line_vi4=arrow3(z,vi4(k,:));line_vi5=arrow3(z,vi5(k,:));  
    text_vi1=text(vi1(k,1)*1.1,vi1(k,2)*1.1,'$v_{\rm{i1}}$','Interpreter', 'Latex','FontSize',32);
    text_vi2=text(vi2(k,1)*1.1,vi2(k,2)*1.1,'$v_{\rm{i2}}$','Interpreter', 'Latex','FontSize',32);
	text_vi3=text(vi3(k,1)*1.1,vi3(k,2)*1.1,'$v_{\rm{i3}}$','Interpreter', 'Latex','FontSize',32);
    text_vi4=text(vi4(k,1)*1.1,vi4(k,2)*1.1,'$v_{\rm{i4}}$','Interpreter', 'Latex','FontSize',32);
    text_vi5=text(vi5(k,1)*1.1,vi5(k,2)*1.1,'$v_{\rm{i5}}$','Interpreter', 'Latex','FontSize',32);
   %synthesis field 
    xi=[vi1(k,1),vi2(k,1),vi3(k,1),vi4(k,1),vi5(k,1),vi1(k,1)];
    yi=[vi1(k,2),vi2(k,2),vi3(k,2),vi4(k,2),vi5(k,2),vi1(k,2)];
    synthesis_field=line(xi,yi,'Color',[0.5 0.5 0.5],'LineWidth',1);    
    %output "p"  
    line_vo1p=arrow3(z,vo1p(k,:));
	point_vo1p=plot(vo1p(k,1),vo1p(k,2),'Marker','o','MarkerEdgeColor','k','MarkerFaceColor','r','MarkerSize',20);
    text_vo1p=text(vo1p(k,1)*1.2,vo1p(k,2)*1.2,'$v_{\rm{o1p}}$','Interpreter', 'Latex','FontSize',32);
   
     
    %figure on the right
    subplot(2,2,[2 4]); 
     %collection of input vectors  
	line_vi1_=arrow3(z,vi1(k,:));line_vi2_=arrow3(z,vi2(k,:));line_vi3_=arrow3(z,vi3(k,:));
    line_vi4_=arrow3(z,vi4(k,:));line_vi5_=arrow3(z,vi5(k,:));  
    text_vi1_=text(vi1(k,1)*1.1,vi1(k,2)*1.1,'$v_{\rm{i1}}$','Interpreter', 'Latex','FontSize',32);
    text_vi2_=text(vi2(k,1)*1.1,vi2(k,2)*1.1,'$v_{\rm{i2}}$','Interpreter', 'Latex','FontSize',32);
	text_vi3_=text(vi3(k,1)*1.1,vi3(k,2)*1.1,'$v_{\rm{i3}}$','Interpreter', 'Latex','FontSize',32);
    text_vi4_=text(vi4(k,1)*1.1,vi4(k,2)*1.1,'$v_{\rm{i4}}$','Interpreter', 'Latex','FontSize',32);
    text_vi5_=text(vi5(k,1)*1.1,vi5(k,2)*1.1,'$v_{\rm{i5}}$','Interpreter', 'Latex','FontSize',32);
   %synthesis field 
    xi=[vi1(k,1),vi2(k,1),vi3(k,1),vi4(k,1),vi5(k,1),vi1(k,1)];
    yi=[vi1(k,2),vi2(k,2),vi3(k,2),vi4(k,2),vi5(k,2),vi1(k,2)];
    synthesis_field_=line(xi,yi,'Color',[0.5 0.5 0.5],'LineWidth',1);
    %output "n"  
    line_vo1n=arrow3(z,vo1n(k,:));
	point_vo1n=plot(vo1n(k,1),vo1n(k,2),'Marker','o','MarkerEdgeColor','k','MarkerFaceColor','b','MarkerSize',20);
    text_vo1n=text(vo1n(k,1)*1.2,vo1n(k,2)*1.2,'$v_{\rm{o1n}}$','Interpreter', 'Latex','FontSize',32);
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % an algorithm
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %input vector coordinates
    vi=[vi1(k,1) vi1(k,2);vi2(k,1) vi2(k,2);vi3(k,1) vi3(k,2);vi4(k,1) vi4(k,2);vi5(k,1) vi5(k,2)];
    %output vector coordinates for the "p" matrix converter
    vop=[vo1p(k,1) vo1p(k,2); vo2p(k,1) vo2p(k,2);vo3p(k,1) vo3p(k,2);vo4p(k,1) vo4p(k,2);vo5p(k,1) vo5p(k,2)];
     %output vector coordinates for the "p" matrix converter
	von=[vo1n(k,1) vo1n(k,2); vo2n(k,1) vo2n(k,2);vo3n(k,1) vo3n(k,2);vo4n(k,1) vo4n(k,2);vo5n(k,1) vo5n(k,2)];    
    
    %calculation the distance between the reference vo1p vector and each vertex
    %M is a number of outputs, N is a number of inputs
    rp=zeros(M,N);  
    Dp=zeros(N,M);
    for m=1:M %for each output vector
        for n=1:N%calculate a distance to each input vector
            rp(m,n)=sqrt((vop(m,1)-vi(n,1))^2+(vop(m,2)-vi(n,2))^2);
        end
        %find a minimum distance and return an index
        [rp_min,index_min_p(m)] =min(rp(m,:));
        %collect candidates among vertices
        vertex1_p(k,m)=index_min_p(m)-1;if (vertex1_p(k,m)<1), vertex1_p(k,m)=vertex1_p(k,m)+N;, end
        vertex2_p(k,m)=index_min_p(m);
        vertex3_p(k,m)=index_min_p(m)+1;if (vertex3_p(k,m)>N), vertex3_p(k,m)=vertex3_p(k,m)-N;, end 
        %prepare the data set
        vi1x=vi(vertex1_p(k,m),1);vi1y=vi(vertex1_p(k,m),2);
        vi2x=vi(vertex2_p(k,m),1);vi2y=vi(vertex2_p(k,m),2);
        vi3x=vi(vertex3_p(k,m),1);vi3y=vi(vertex3_p(k,m),2);
        vo1x=vop(m,1); vo1y=vop(m,2);
        %calculation of the PWM duty cycle
        sref=abs((vi2x-vi1x)*(vi3y-vi1y)-(vi2y-vi1y)*(vi3x-vi1x));
        d1p(k,m)=abs(((vi2x-vo1x)*(vi3y-vo1y)-(vi2y-vo1y)*(vi3x-vo1x)))/sref;
        d2p(k,m)=abs(((vi1x-vo1x)*(vi3y-vo1y)-(vi1y-vo1y)*(vi3x-vo1x)))/sref;
        d3p(k,m)= 1-d1p(k,m)-d2p(k,m);
        %fill the global PWM table for CMC_P
        Dp(vertex1_p(k,m),m)=d1p(k,m);
        Dp(vertex2_p(k,m),m)=d2p(k,m);
        Dp(vertex3_p(k,m),m)=d3p(k,m);
    end
    
    %calculation the distance between the reference vo1p vector and each vertex
    %M is a number of outputs, N is a number of inputs
    rn=zeros(M,N);    
    Dn=zeros(N,M);
    for m=1:M %for each output vector
        for n=1:N%calculate a distance to each input vector
            rn(m,n)=sqrt((von(m,1)-vi(n,1))^2+(von(m,2)-vi(n,2))^2);
        end
        %find a minimum distance and return an index
        [rn_min, index_min_n(m)] =min(rn(m,:));
        %collect candidates among vertices
        vertex1_n(k,m)=index_min_n(m)-1;if (vertex1_n(k,m)<1), vertex1_n(k,m)=vertex1_n(k,m)+N; end
        vertex2_n(k,m)=index_min_n(m);
        vertex3_n(k,m)=index_min_n(m)+1;if (vertex3_n(k,m)>N), vertex3_n(k,m)=vertex3_n(k,m)-N; end 
       %prepare the data set
        vi1x=vi(vertex1_n(k,m),1);vi1y=vi(vertex1_n(k,m),2);
        vi2x=vi(vertex2_n(k,m),1);vi2y=vi(vertex2_n(k,m),2);
        vi3x=vi(vertex3_n(k,m),1);vi3y=vi(vertex3_n(k,m),2);
        vo1x=von(m,1); vo1y=von(m,2);
        %calculation of the PWM duty cycle
        sref=abs((vi2x-vi1x)*(vi3y-vi1y)-(vi2y-vi1y)*(vi3x-vi1x));
        d1n(k,m)=abs(((vi2x-vo1x)*(vi3y-vo1y)-(vi2y-vo1y)*(vi3x-vo1x)))/sref;
        d2n(k,m)=abs(((vi1x-vo1x)*(vi3y-vo1y)-(vi1y-vo1y)*(vi3x-vo1x)))/sref;
        d3n(k,m)= 1-d1n(k,m)-d2n(k,m);
        %fill the global PWM table for CMC_N
        Dn(vertex1_n(k,m),m)=d1n(k,m);
        Dn(vertex2_n(k,m),m)=d2n(k,m);
        Dn(vertex3_n(k,m),m)=d3n(k,m);
    end
    
    %figure on the left
    subplot(2,2,[1 3]);
    xi=[vi(vertex1_p(k,1),1) vi(vertex2_p(k,1),1) vi(vertex3_p(k,1),1) vi(vertex1_p(k,1),1)];
    yi=[vi(vertex1_p(k,1),2) vi(vertex2_p(k,1),2) vi(vertex3_p(k,1),2) vi(vertex1_p(k,1),2)];
    triangle_p=line(xi,yi,'Color',[0.6 0.6 0.6],'LineWidth',6);
   %figure on the right
    subplot(2,2,[2 4]);
    xi=[vi(vertex1_n(k,1),1) vi(vertex2_n(k,1),1) vi(vertex3_n(k,1),1) vi(vertex1_n(k,1),1)];
    yi=[vi(vertex1_n(k,1),2) vi(vertex2_n(k,1),2) vi(vertex3_n(k,1),2) vi(vertex1_n(k,1),2)];
    triangle_n=line(xi,yi,'Color',[0.6 0.6 0.6],'LineWidth',6);
    
    %average voltage
	vo_avg_p(k,:)=(vi(:,1)')*Dp;
	vo_avg_n(k,:)=(vi(:,1)')*Dn;
    vo_avg(k,:)=vo_avg_p(k,:)-vo_avg_n(k,:);
     
   %the load current generator based on the IIR simple filter
    if(k>1)
         for m=1:M
              io(k,m) = io(k-1,m)+((vo_avg_p(k,m)-vo_avg_n(k,m))/10 - 0.15*io(k-1,m));
         end       
    end
    
    %average input currents
    ii_avg_p(k,:)=io(k,:)*(Dp');
    ii_avg_n(k,:)=-io(k,:)*(Dn');
	ii_avg(k,:)=ii_avg_p(k,:)+ii_avg_n(k,:);
    
    %animation tusk
    drawnow;   
    if k==len
        break
    end
    %%%%%%%%%%%%%%%% clear graphical objects in left drawings %%%%%%%%%%%%%%
    delete(line_vi1);
    delete(line_vi2);delete(line_vi3);delete(line_vi4);delete(line_vi5);
    delete(text_vi1);delete(text_vi2);delete(text_vi3);delete(text_vi4);delete(text_vi5);
    delete(synthesis_field);    
    delete(point_vo1p);delete(line_vo1p);delete(text_vo1p);  
    delete(triangle_p);
     %%%%%%%%%%%%%%%% clear graphical objects in left drawings %%%%%%%%%%%%%%
    delete(line_vi1_);delete(line_vi2_);delete(line_vi3_);delete(line_vi4_);delete(line_vi5_);
    delete(text_vi1_);delete(text_vi2_);delete(text_vi3_);delete(text_vi4_);delete(text_vi5_);
    delete(synthesis_field_);
    delete(point_vo1n);delete(line_vo1n);delete(text_vo1n); 
    delete(triangle_n);
end

 
%second window with simulation results
fig2=figure('units','normalized','outerposition',[0 0 0.9 0.9]);
name='Direct Modulation concept for Conventional Matrix Converter with 3 inputs and 3 outputs: figure 2/2';
set(fig2,'Color','white','MenuBar','figure','Name',name,'NumberTitle','off');
fontsize_axes=24;
fontsize_label=24;
 
sb1=subplot(311);plot(t,(vo_avg_p(:,1)-vo_avg_n(:,1)),t,io(:,1),'LineWidth',2);
title('Output waveform in p.u.','FontSize',8,'Interpreter','Latex'); 
xlim([0 tend]);ylim([-2 2]);grid on;
set(get(gcf,'CurrentAxes'),'FontName','Times New Roman','FontSize',fontsize_axes,'YTick',[-2:1:2]);
legend('$v_{\rm{o1}}$','$i_{\rm{o1}}$','FontSize',24,'Interpreter','Latex');

subplot(312);plot(t,[vi1(:,1) ii_avg(:,1) ii_avg_p(:,1) ii_avg_n(:,1)]','LineWidth',2);
title('Input waveforms in p.u.','FontSize',8,'Interpreter','Latex'); 
xlim([0 tend]);ylim([-3 3]);grid on;
set(get(gcf,'CurrentAxes'),'FontName','Times New Roman','FontSize',fontsize_axes,'YTick',[-3:1.5:3]);
legend('$v_{\rm{i1}}$','$i_{\rm{i1}}$','$i_{\rm{i1p}}$','$i_{\rm{i1n}}$','FontSize',24,'Interpreter','Latex');

subplot(313);plot(t,[d1p(:,1) d2p(:,1) d3p(:,1) ]','LineWidth',2);grid on;
 title('PWM duty cycles: $d_{1p}$, $d_{2p}$, $d_{3p}$','FontSize',8,'Interpreter','Latex'); 
xlim([0 tend]);ylim([0 1]);grid on;
set(get(gcf,'CurrentAxes'),'FontName','Times New Roman','FontSize',fontsize_axes,'YTick',[0:0.25:1]);
legend('$d_{1p}$', '$d_{2p}$', '$d_{3p}$','FontSize',24,'Interpreter','Latex');
xlabel('{t [s]}','FontSize',fontsize_label,'FontName','Times New Roman');

disp('====================================================================================================================================');
disp('Thanks for watching.');
disp('====================================================================================================================================');

